//Sample test program for MSP430-5438STK
//Test RS
//Return echo ot RS

#include "io430.h"

#define OSC_FREQ 18000000
#define BAUDRATE     9600

void CLOCK_XT2_Init()
{
   int timeout = 5000;
   P11DIR = P11DIR | 0x02 | 0x04;            //set P11.1 & P11.2 as output
   P11SEL = P11SEL | 0x02 | 0x04;            //set P11.1 & P11.2 as MCLK/SMCLK function
   P5SEL = P5SEL | 0x04 | 0x08;              //set P5.2 & P5.3 as XT2IN/XT2OUT

   UCSCTL6 = UCSCTL6 & (~XT2OFF);            //set XT2OFF=0 (bit8 = 0) --> enable XT2
   UCSCTL3 = UCSCTL3 | 0x20;                 //set SELREF = 010 --> FLL clock source = REFOCLK
   UCSCTL4 = UCSCTL4 | 0x200;                //set SELA = 010 --> ACLK=REFO

   do
   {
     if (!(UCSCTL7 & 0x08)) break;
     UCSCTL7 = UCSCTL7 & ~(0x0F);            // Clear XT2,XT1,DCO fault flags
     SFRIFG1 = SFRIFG1 & ~(0x02);            // Clear fault flags (SFRIFG1.OFIFG = 0)
     timeout--;
   }
   while ((SFRIFG1 & 0x02) && timeout);      // Test oscillator fault flag

   UCSCTL5 = UCSCTL5 | 0x0005;               //MCLK/SMCLK divisor = 32

   UCSCTL4 = UCSCTL4 | 0x55;                 //SMCLK=MCLK=XT2
   UCSCTL4 = ((UCSCTL4 & ~0x100) & ~0x200) & ~0x400;     //ACLK=XT1  (SELA = 0b000)
}


void UART2_Init(unsigned long baud)
{
    float n, x;
    P9DIR = P9DIR | 1<<4;           //set P9.4=1 (Transmitter) as output
    P9DIR = P9DIR & ~(1<<5);        //set P9.5=0 (Receiver) as input
    P9SEL = P9SEL | 1<<4 | 1<<5;    //set P9.4=1 & P9.5=1 as TX/RX


    UCA2CTL1 = UCA2CTL1 | 0x01;     //UCSWRST = 1 (bit0 = 1)   //page 417

    UCA2CTL1 = (UCA2CTL1 | 1<<7) & ~(1<<6);       //select SMCLK as BaudRate source clock (bit6 = 1; bit7 = 1)

    //calculate registers for the BaudRate +

    UCA2MCTL = UCA2MCTL | 0x01;       //UCOS16 = 1
    //calculate UCBR
    n = OSC_FREQ / baud;
    UCA2BR0 = (int)(n/16) % 256;
    UCA2BR1 = (int)(n/16) / 256;

    //caluclate UCBRS
    x = (int)((n-(int)(n))*8+0.5);
    UCA2MCTL = UCA2MCTL | (int)(x)<<1;

    //caluclate UCBRF
    x = (int)(((n/16 - (int)(n/16))*16) + 0.5);
    UCA2MCTL = UCA2MCTL | (int)(x)<<4;

    //calculate registers for the BaudRate -

    UCA2CTL0 = 0x00;                  //Set Format Frame: 8 bit data; no parity; 1 stop bit

    UCA2CTL1 = UCA2CTL1 & ~(0x01);    //UCSWRST = 0 (bit0 = 0) //page 417
}

unsigned char UART2_Receive()
{
    if (UCA2IFG & 0x01)
      return UCA2RXBUF;
    else
      return 0;
}

void UART2_Transmit (unsigned char data)
{
   while (!(UCA2IFG & 0x02));
   UCA2TXBUF = data;
}

int main( void )
{
  // Stop watchdog timer to prevent time out reset
  WDTCTL = WDTPW + WDTHOLD;
  char c;

  CLOCK_XT2_Init();
  UART2_Init(BAUDRATE);

  while (1)
  {
	c = UART2_Receive();
	if (c)
		UART2_Transmit(c+1);
  }
}
